@@ -48,9 +48,8 @@ tbl$addData(tocsummary, firstColumnAsRowHeaders=TRUE,
             columnFormats=columnFormats)
 
 # get the cells and apply styling
-highlight <- tbl$createInlineStyle(declarations=list("background-color"="#FFCC66"))
-cells <- tbl$getCells(specifyCellsAsList=TRUE, rowNumbers=c(1, 3))
-lst <- lapply(cells, function(cell) {cell$style <- highlight})
+cells <- tbl$getCells(rowNumbers=c(1, 3))
+tbl$setStyling(cells=cells, declarations=list("background-color"="#FFCC66"))
 tbl$renderTable()
 
 ## ---- message=FALSE, warning=FALSE---------------------------------------
@@ -78,9 +77,8 @@ tbl$addData(tocsummary, firstColumnAsRowHeaders=TRUE,
             columnFormats=columnFormats)
 
 # get the cells and apply styling
-highlight <- tbl$createInlineStyle(declarations=list("background-color"="#FFCC66"))
-cells <- tbl$getCells(specifyCellsAsList=TRUE, columnNumbers=2)
-lst <- lapply(cells, function(cell) {cell$style <- highlight})
+cells <- tbl$getCells(columnNumbers=2)
+tbl$setStyling(cells=cells, declarations=list("background-color"="#FFCC66"))
 tbl$renderTable()
 
 ## ---- message=FALSE, warning=FALSE---------------------------------------
@@ -108,9 +106,8 @@ tbl$addData(tocsummary, firstColumnAsRowHeaders=TRUE,
             columnFormats=columnFormats)
 
 # get the cells and apply styling
-highlight <- tbl$createInlineStyle(declarations=list("background-color"="#FFCC66"))
-cells <- tbl$getCells(specifyCellsAsList=TRUE, cellCoordinates=list(c(2, 3)))
-lst <- lapply(cells, function(cell) {cell$style <- highlight})
+cells <- tbl$getCells(cellCoordinates=list(c(2, 3)))
+tbl$setStyling(cells=cells, declarations=list("background-color"="#FFCC66"))
 cat("The raw value of the cell is", cells[[1]]$rawValue, "and the formatted value is", cells[[1]]$formattedValue, ".")
 tbl$renderTable()
 
@@ -139,9 +136,8 @@ tbl$addData(tocsummary, firstColumnAsRowHeaders=TRUE,
             columnFormats=columnFormats)
 
 # get the cells and apply styling
-highlight <- tbl$createInlineStyle(declarations=list("background-color"="#FFCC66"))
-cells <- tbl$getCells(specifyCellsAsList=TRUE, cellCoordinates=list(c(2, 3), c(3, 4), c(5, 6)))
-lst <- lapply(cells, function(cell) {cell$style <- highlight})
+cells <- tbl$getCells(cellCoordinates=list(c(2, 3), c(3, 4), c(5, 6)))
+tbl$setStyling(cells=cells, declarations=list("background-color"="#FFCC66"))
 tbl$renderTable()
 
 ## ---- message=FALSE, warning=FALSE---------------------------------------
@@ -169,9 +165,8 @@ tbl$addData(tocsummary, firstColumnAsRowHeaders=TRUE,
             columnFormats=columnFormats)
 
 # get the cells and apply styling
-highlight <- tbl$createInlineStyle(declarations=list("background-color"="#FFCC66"))
-cells <- tbl$getCells(specifyCellsAsList=TRUE, rowNumbers=2, columnNumbers=4, cellCoordinates=list(c(5, 6)))
-lst <- lapply(cells, function(cell) {cell$style <- highlight})
+cells <- tbl$getCells(rowNumbers=2, columnNumbers=4, cellCoordinates=list(c(5, 6)))
+tbl$setStyling(cells=cells, declarations=list("background-color"="#FFCC66"))
 tbl$renderTable()
 
 ## ---- message=FALSE, warning=FALSE---------------------------------------
@@ -199,9 +194,8 @@ tbl$addData(tocsummary, firstColumnAsRowHeaders=TRUE,
             columnFormats=columnFormats)
 
 # get the cells and apply styling
-highlight <- tbl$createInlineStyle(declarations=list("background-color"="#00FF00"))
 cells <- tbl$getCells(specifyCellsAsList=FALSE, rowNumbers=c(1, 3))
-lst <- lapply(cells, function(cell) {cell$style <- highlight})
+tbl$setStyling(cells=cells, declarations=list("background-color"="#00FF00"))
 tbl$renderTable()
 
 ## ---- message=FALSE, warning=FALSE---------------------------------------
@@ -229,9 +223,8 @@ tbl$addData(tocsummary, firstColumnAsRowHeaders=TRUE,
             columnFormats=columnFormats)
 
 # get the cells and apply styling
-highlight <- tbl$createInlineStyle(declarations=list("background-color"="#00FF00"))
 cells <- tbl$getCells(specifyCellsAsList=FALSE, columnNumbers=2)
-lst <- lapply(cells, function(cell) {cell$style <- highlight})
+tbl$setStyling(cells=cells, declarations=list("background-color"="#00FF00"))
 tbl$renderTable()
 
 ## ---- message=FALSE, warning=FALSE---------------------------------------
@@ -259,9 +252,8 @@ tbl$addData(tocsummary, firstColumnAsRowHeaders=TRUE,
             columnFormats=columnFormats)
 
 # get the cells and apply styling
-highlight <- tbl$createInlineStyle(declarations=list("background-color"="#00FF00"))
 cells <- tbl$getCells(specifyCellsAsList=FALSE, rowNumbers=2, columnNumbers=3)
-lst <- lapply(cells, function(cell) {cell$style <- highlight})
+tbl$setStyling(cells=cells, declarations=list("background-color"="#00FF00"))
 cat("The raw value of the cell is", cells[[1]]$rawValue, "and the formatted value is", cells[[1]]$formattedValue, ".")
 tbl$renderTable()
 
@@ -290,9 +282,8 @@ tbl$addData(tocsummary, firstColumnAsRowHeaders=TRUE,
             columnFormats=columnFormats)
 
 # get the cells and apply styling
-highlight <- tbl$createInlineStyle(declarations=list("background-color"="#00FF00"))
 cells <- tbl$getCells(specifyCellsAsList=FALSE, rowNumbers=c(2, 3, 5), columnNumbers=c(3, 4, 6))
-lst <- lapply(cells, function(cell) {cell$style <- highlight})
+tbl$setStyling(cells=cells, declarations=list("background-color"="#00FF00"))
 tbl$renderTable()
 
 ## ---- message=FALSE, warning=FALSE---------------------------------------
@@ -320,9 +311,8 @@ tbl$addData(tocsummary, firstColumnAsRowHeaders=TRUE,
             columnFormats=columnFormats)
 
 # get the cells and apply styling
-highlight <- tbl$createInlineStyle(declarations=list("background-color"="#00FF00"))
 cells <- tbl$getCells(specifyCellsAsList=FALSE, rowNumbers=c(2, NA, 5), columnNumbers=c(NA, 4, 6))
-lst <- lapply(cells, function(cell) {cell$style <- highlight})
+tbl$setStyling(cells=cells, declarations=list("background-color"="#00FF00"))
 tbl$renderTable()
 
 ## ---- message=FALSE, warning=FALSE---------------------------------------
@@ -350,9 +340,8 @@ tbl$addData(tocsummary, firstColumnAsRowHeaders=TRUE,
             columnFormats=columnFormats)
 
 # apply the formatting
-redStyle <- tbl$createInlineStyle(declarations=list("background-color"="#FFC7CE", "color"="#9C0006"))
 cells <- tbl$findCells(columnNumbers=5:6, minValue=0, maxValue=40, includeNull=FALSE, includeNA=FALSE)
-lst <- lapply(cells, function(cell) {cell$style <- redStyle})
+tbl$setStyling(cells=cells, declarations=list("background-color"="#FFC7CE", "color"="#9C0006"))
 tbl$renderTable()
 
 ## ---- message=FALSE, warning=FALSE---------------------------------------
@@ -380,16 +369,13 @@ tbl$addData(tocsummary, firstColumnAsRowHeaders=TRUE,
             columnFormats=columnFormats)
 
 # apply the red formatting
-redStyle <- tbl$createInlineStyle(declarations=list("background-color"="#FFC7CE", "color"="#9C0006"))
 cells <- tbl$findCells(columnNumbers=5:6, minValue=0, maxValue=40, includeNull=FALSE, includeNA=FALSE)
-lst <- lapply(cells, function(cell) {cell$style <- redStyle})
+tbl$setStyling(cells=cells, declarations=list("background-color"="#FFC7CE", "color"="#9C0006"))
 # apply the yellow formatting
-yellowStyle <- tbl$createInlineStyle(declarations=list("background-color"="#FFEB9C", "color"="#9C5700"))
 cells <- tbl$findCells(columnNumbers=5:6, minValue=40, maxValue=60, includeNull=FALSE, includeNA=FALSE)
-lst <- lapply(cells, function(cell) {cell$style <- yellowStyle})
+tbl$setStyling(cells=cells, declarations=list("background-color"="#FFEB9C", "color"="#9C5700"))
 # apply the green formatting
-greenStyle <- tbl$createInlineStyle(declarations=list("background-color"="#C6EFCE", "color"="#006100"))
 cells <- tbl$findCells(columnNumbers=5:6, minValue=60, maxValue=100, includeNull=FALSE, includeNA=FALSE)
-lst <- lapply(cells, function(cell) {cell$style <- greenStyle})
+tbl$setStyling(cells=cells, declarations=list("background-color"="#C6EFCE", "color"="#006100"))
 tbl$renderTable()
 
